PROGRAM MAXLEN (filein, fileout);
(***************************************************************************
*  Written in TURBO Pascal, version 4.0, by Carol N. Gerlitz, August 1989:
*  this program reads references from a delimited database file for DBASE
*  and writes a file that lists maximum lengths for each field in the file
***************************************************************************)
uses CRT;

type  intarray = array[1..5] of integer;
      longstr = array[1..1275] of char;

var
  filein  : text;       (* input data file *)
  fileout : text;       (* final output data file *)
  infile  : string[14]; (* user-input data file name assigned to filein *)
  i,n     : integer;    (* indices: n counts references *)
  longest : intarray;   (* longest length of each field in database *)

PROCEDURE PAUSE (x,y : byte; msg : string);
(**************************************************************************)
var ch : char;
begin
 GotoXY(x,y);
 write(msg, ' . . . ');
 ch := ReadKey;
end; (* pause *)

FUNCTION VALIDNAME : string;
(***************************************************************************
* Reads user-entered filename; checks to be sure it hasn't been left blank
***************************************************************************)
var firstpass : boolean;
    filename : string[14];
begin (* validname *)
 firstpass := true;
 repeat (* until file name is entered *)
  readln(filename);
  if filename = '' then
   if firstpass then
    begin
     write(^G, ' ==> Please enter a valid file name:  ');
     firstpass := false;
    end
   else (* if not firstpass *)
    begin
     GotoXY(39,WhereY-1);
     ClrEOL;
     write(^G);
    end;
 until filename <> '';
 ValidName := filename;
end; (* validname *)

PROCEDURE INITIALIZE (var line : longstr);
(***************************************************************************
* Initializes the temporary one-line record
***************************************************************************)
var i : integer;
begin (* initialize *)
 for i := 1 to 1275 do
  line[i] := ' ';
end; (* initialize *)

PROCEDURE ODOMETER (n,x : integer; msg1, msg2 : string);
(***************************************************************************
* write odometer to screen
***************************************************************************)
begin
 if n = 1 then
  begin
   GotoXY(7,7);
   write(msg1,'....');
   GotoXY(15,10);
   write(msg2, n:4);
  end
 else (* if n > 1 *)
  begin
   GotoXY(x,10);
   write(n:4);
  end;
end; (* odometer *)


PROCEDURE READLINE (var line : longstr; var linelength : integer);
(***************************************************************************
* Reads one line from the temporary file and assigns it to long string array
***************************************************************************)
var i : integer; (* index *)
    len1,len2,len3,len4,len5 : byte;
                            (* lengths of line1, line2, line3, line4,line5 *)
    line1,line2,line3,line4,line5 : string[255];
          (* read long line from temporary file in parts *)

begin
 (* read in one record of data at a time *)
 repeat   (* skip blank lines *)
  read(filein, line1);
  if line1 = '' then
   readln;
 until line1 <> '';
 readln(filein,line2,line3,line4,line5);
 len1 := length(line1);
 len2 := length(line2);
 len3 := length(line3);
 len4 := length(line4);
 len5 := length(line5);
 linelength := len1 + len2 + len3 + len4 + len5;

 (* assign line segments to one long string *)
 for i := 1 to len1 do
  line[i] := line1[i];
 if len2 <> 0 then
  begin
   for i := 1 to len2 do
    line[i+255] := line2[i];
   if len3 <> 0 then
    begin
     for i := 1 to len3 do
      line[i+510] := line3[i];
     if len4 <> 0 then
      begin
       for i := 1 to len4 do
        line[i+765] := line4[i];
       if len5 <> 0 then
        for i := 1 to len5 do
         line[i+1020] := line5[i];
      end; (* if len4 <> 0 *)
    end; (* if len3 <> 0 *)
  end; (* if len2 <> 0 *)
end; (* readline *)

PROCEDURE CHECKDATA (n : integer; var fieldlength : intarray);
(***************************************************************************
* Reads references from database and finds longest length for each field
*  in database file
***************************************************************************)
var i,j          : integer; (* faithful index friends *)
    line         : longstr;
    linelength   : integer; (* length of line (len1+len2+len3+len4+len5) *)
    startfield   : integer; (* counts start of each database field *)

begin (* checkdata *)
 Initialize(line);   (* initialize the record *)

 startfield := 2;
 j := 0;
 linelength := 0;

 Odometer(n, 43, 'EVALUATING FIELD LENGTHS OF REFERENCES FOR DATABASE FILE',
          'EVALUATING REFERENCE NUMBER ');
 ReadLine(line, linelength);

 (* calculate length of database fields *)
 for i := 4 to linelength do
  if i < linelength then
   begin
    if ((line[i] = '"') and (line[i-1] = ',') and (line[i-2]='"')) then
     begin
      j := j + 1;
      fieldlength[j] := i - 2 - startfield;
      startfield := i + 1;
     end;
   end (* if i < linelength *)
  else if i = linelength then
   begin
    fieldlength[5] := linelength - startfield;
   end;
end; (* checkdata *)

PROCEDURE WRITEFILE (longest, number : intarray);
(***************************************************************************
* Writes maximum length of each database field to output file
***************************************************************************)
var i : integer; (* index *)

begin
 writeln(fileout); writeln(fileout);
 writeln(fileout, 'FOR DATABASE FILE ', infile,':');
 write(fileout, '------------------');
 for i := 1 to length(infile) do
  write(fileout, '-');
 writeln(fileout);
 writeln(fileout);
 writeln(fileout, 'Longest AUTHOR is ', longest[1]:1,
   ' characters  (Ref # ', number[1]:1, ')');
 writeln(fileout);
 writeln(fileout, 'Longest YEAR is ', longest[2]:2,
   ' characters  (Ref # ', number[2]:1, ')');
 writeln(fileout);
 writeln(fileout, 'Longest TITLE1 (first title field) is ', longest[3]:1,
   ' characters  (Ref # ', number[3]:1, ')');
 writeln(fileout);
 writeln(fileout, 'Longest TITLE2 (second title field) is ', longest[4]:1,
   ' characters  (Ref # ', number[4]:1, ')');
 writeln(fileout);
 writeln(fileout, 'Longest CITATION is ', longest[5]:1,
   ' characters  (Ref # ', number[5]:1, ')');
end; (* writefile *)

PROCEDURE OPENFILES;
(***************************************************************************
* Prompts user for input file name; opens data files
***************************************************************************)
begin
 (* prompt user for and read in input and output file names *)
 writeln('ENTER NAME OF INPUT DATA FILE (delimited database file)');
 write('   Input file?  ');
 infile := Validname;
 writeln;

 assign(filein,infile);  (* open input data file *)
 reset(filein);

 assign(fileout,'MAX.DAT');  (* open output database file *)
 rewrite(fileout);
end; (* openfiles *)

PROCEDURE CLOSEFILES;
(***************************************************************************
* Closes data files
***************************************************************************)
begin
 close(filein);
 close(fileout);
end; (* closefiles *)

(**************************************************************************)

var fieldlength : intarray; (* length of each field in current reference *)
    number : intarray; (* reference number of longest length for each field *)
Begin  (* MaxLen *)

 (* print information message *)
 ClrScr;  GotoXY(1,7);
 writeln('WELCOME TO Program MaxLen--');
 writeln('  this program will read your delimited database file for DBASE');
 writeln('  or PARADOX (output by PROGRAM REFS2DB) and write out the ');
 writeln('  maximum length for each field in the database file.');
 writeln;

 OpenFiles;
 n := 0;
 for i := 1 to 5 do
  longest[i] := 0;

 (* evaluate references in database file *)
 ClrScr;
 while not eof(filein) do
  begin
   n := n + 1;
   for i := 1 to 5 do
    fieldlength[i] := 0;
   CheckData(n,fieldlength);
{
WRITE(fileout,'AUTHOR ', n:1,': ', fieldlength[1]:1,'; YEAR: ',fieldlength[2]:1);
WRITE(fileout,'; TITLE1: ',fieldlength[3]:1,'; TITLE2: ',fieldlength[4]:1);
WRITELN(fileout,'; CITATION: ',fieldlength[5]:1);
}
   for i := 1 to 5 do
    if fieldlength[i] > longest[i] then
     begin
      longest[i] := fieldlength[i];
      number[i] := n;
     end; (* for i := 1 to 5 *)
  end; (* while not eof *)

 WriteFile(longest,number);

 (* print warning if unread data remains in file *)
 if not eof (filein) then
  begin
   writeln('==> Entire data file was not read....remaining data ignored.');
   writeln('    Last reference read is number ', n:1);
   Pause(7,21,'Press any key to continue');
  end; (* if *)

 GotoXY(7,14);
 write('NUMBER OF REFERENCES EVALUATED IN DATABASE FILE:  ', n:1);
 GotoXY(7,16);
 write('YOUR FILE WITH MAXIMUM LENGTH INFO, "MAX.DAT", IS READY');
 Pause(7,21,'Press any key to exit to DOS');

 CloseFiles;
 ClrScr;
End.  (* MaxLen *)
