PROGRAM REFS2PAR (filein, tempfile, fileout);
(***************************************************************************
*  Written in TURBO Pascal, version 4.0, by Carol N. Gerlitz, July 1989:
*  this program reads references from an ascii file and outputs a delimited
*  data file that can be imported into PARADOX; maximum length of first
*  TITLE data field will be 255 characters
***************************************************************************)
uses CRT;

type    longstr = array[1..1020] of char;

var
  filein    : text;       (* input data file *)
  fileout   : text;       (* final output data file *)
  i,n : integer;
  infile    : string[14]; (* user-input data file name assigned to filein *)
  number    : integer;    (* counts number of references in data file *)
  outfile   : string[14]; (* user-input data file name assigned to fileout *)
  tempfile  : text;       (* temporary data file *)

PROCEDURE PAUSE (x,y : byte; msg : string);
(**************************************************************************)
var ch : char;
begin
 GotoXY(x,y);
 write(msg, ' . . . ');
 ch := ReadKey;
end; (* pause *)

FUNCTION VALIDNAME : string;
(***************************************************************************
* Reads user-entered filename; checks to be sure it hasn't been left blank
***************************************************************************)
var firstpass : boolean;
    filename : string[14];
begin (* validname *)
 firstpass := true;
 repeat (* until file name is entered *)
  readln(filename);
  if filename = '' then
   if firstpass then
    begin
     write(^G, ' ==> Please enter a valid file name:  ');
     firstpass := false;
    end
   else (* if not firstpass *)
    begin
     GotoXY(39,WhereY-1);
     ClrEOL;
     write(^G);
    end;
 until filename <> '';
 ValidName := filename;
end; (* validname *)

PROCEDURE INITIALIZE (var line : longstr);
(***************************************************************************
* Initializes the temporary one-line record
***************************************************************************)
begin (* initialize *)
 for i := 1 to 1020 do
  line[i] := ' ';
end; (* initialize *)

PROCEDURE ODOMETER (n,x : integer; msg1, msg2 : string);
(***************************************************************************
* write odometer to screen
***************************************************************************)
begin
 if n = 1 then
  begin
   GotoXY(10,7);
   write(msg1,'....');
   GotoXY(15,10);
   write(msg2, n:4);
  end
 else (* if n > 1 *)
  begin
   GotoXY(x,10);
   write(n:4);
  end;
end; (* odometer *)

PROCEDURE READHANGING (var filein, tempfile : text; var number : integer);
(***************************************************************************
* Reads lines from original ascii file of references in hanging indent
* paragraph format (with or without blank line between references) and
* writes each ref to a single line in a temporary file
***************************************************************************)
var i,firstchar : integer;     (* faithful index friend *)
    line        : string[80];  (* reads one line of file at a time *)
    firstref    : boolean;  (* true until first line of first ref is read *)

begin (* readhanging *)
 ClrScr;
 firstref := true;
 while not (eof (filein)) do
  begin
   repeat
    readln(filein, line);
   until length(line) > 0;
   if (line[1] <> ' ') or (line[2] <> ' ') then
    begin
     if firstref then (* if first reference in file, ignore writeln *)
      firstref := false
     else (* start new line for new record *)
      writeln(tempfile);
     number := number + 1;
     Odometer(number, 40, 'READING REFERENCES FROM ORIGINAL ASCII FILE',
              'READING REFERENCE NUMBER ');
    end; (* if line[1] <> ' ' *)

   i := 0;
   firstchar := 0;
   repeat (* until firstchar <> 0 *)
    i := i + 1;
    if line[i] <> ' ' then
     firstchar := i;
   until firstchar <> 0;
   for i := firstchar to length(line) do
    write(tempfile, line[i]);

   (* if no space at end of line, add one *)
   if line[length(line)] <> ' ' then
    write(tempfile, ' ');

  end; (* while not eof *)

 if number = 0 then
  writeln(' DATA FILE WAS NOT READ...NUMBER IS STILL ', number:1);

 (* print warning if unread data remains in file *)
 if not eof (filein) then
  begin (* if *)
   writeln('==> Entire data file was not read....remaining data ignored.');
   writeln('    Last reference read is number ', number:1);
  end; (* if *)

end; (* readhanging *)

PROCEDURE READFLUSH (var filein, tempfile : text; var number : integer);
(***************************************************************************
* Reads lines from original ascii file of references in flush left paragraph
* format (with single blank line between references) and writes each ref to
* a single line in a temporary file
***************************************************************************)
var i,firstchar : integer;
    line      : string[80]; (* reads one line of file at a time *)
    firstline : boolean;    (* true until first line of each ref is read *)
    firstref  : boolean;    (* true until first line of first ref is read *)

begin (* readflush *)
 ClrScr;
 firstref := true;
 firstline := true;
 while not (eof (filein)) do
  begin
   repeat
    readln(filein, line);
    if length(line) = 0 then
     firstline := true;
   until length(line) > 0;
   if firstline then
    begin
     firstline := false;
     if firstref then (* if first reference in file, ignore writeln *)
      firstref := false
     else (* start new line for new record *)
      writeln(tempfile);
     number := number + 1;
     Odometer(number, 40, 'READING REFERENCES FROM ORIGINAL ASCII FILE',
              'READING REFERENCE NUMBER ');
    end; (* if line[1] <> ' ' *)

   i := 0;
   firstchar := 0;
   repeat (* until firstchar <> 0 *)
    i := i + 1;
    if line[i] <> ' ' then
     firstchar := i;
   until firstchar <> 0;
   for i := firstchar to length(line) do
    write(tempfile, line[i]);

   (* if no space at end of line, add one *)
   if line[length(line)] <> ' ' then
    write(tempfile, ' ');

  end; (* while not eof *)

 if number = 0 then
  writeln(' DATA FILE WAS NOT READ...NUMBER IS STILL ', number:1);

 (* print warning if unread data remains in file *)
 if not eof (filein) then
  begin (* if *)
   writeln('==> Entire data file was not read....remaining data ignored.');
   writeln('    Last reference read is number ', number:1);
  end; (* if *)

end; (* readflush *)

PROCEDURE READSTANDARD (var filein, tempfile : text; var number : integer);
(***************************************************************************
* Reads lines from original ascii file of references in standard indent
* paragraph format (with or without blank line between references) and
* writes each ref to a single line in a temporary file
***************************************************************************)
var i,firstchar : integer;     (* faithful index friend *)
    line        : string[80];  (* reads one line of file at a time *)
    firstref    : boolean;

begin (* readstandard *)
 ClrScr;
 firstref := true;
 while not (eof (filein)) do
  begin
   repeat
    readln(filein, line);
   until length(line) > 0;
   if (line[1] = ' ') and (line[2] = ' ') then
   (* start new line for new record *)
    begin
     if firstref then
      firstref := false
     else
      writeln(tempfile);
     number := number + 1;
     Odometer(number, 40, 'READING REFERENCES FROM ORIGINAL ASCII FILE',
              'READING REFERENCE NUMBER ');
    end; (* if line[1] = ' ' *)

   i := 0;
   firstchar := 0;
   repeat (* until firstchar <> 0 *)
    i := i + 1;
    if line[i] <> ' ' then
     firstchar := i;
   until firstchar <> 0;
   for i := firstchar to length(line) do
    write(tempfile, line[i]);

   (* if no space at end of line, add one *)
   if line[length(line)] <> ' ' then
    write(tempfile, ' ');

  end; (* while not eof *)

 if number = 0 then
  writeln(' DATA FILE WAS NOT READ...NUMBER IS STILL ', number:1);

 (* print warning if unread data remains in file *)
 if not eof (filein) then
  begin (* if *)
   writeln('==> Entire data file was not read....remaining data ignored.');
   writeln('    Last reference read is number ', number:1);
  end; (* if *)

end; (* readstandard *)

FUNCTION MENU : char;
var m : char;
begin
 GotoXY(10,9);
 writeln('WHAT IS THE FORMAT OF YOUR REFERENCE FILE?');
 writeln;
 writeln('            1.  Hanging indent');
 writeln('            2.  Flush left');
 writeln('            3.  Standard indent');
 GotoXY(10,15);
 write('Enter the menu number of your choice:  ');
 repeat (* until a valid choice is made from the menu *)
  m := ReadKey;
  if not (m in ['1'..'3']) then
   begin
    GotoXY(10,17);
    write(^G, '==> Please enter the number of your choice:  ');
   end;
 until m in ['1'..'3'];
 Menu := m;
end; (* menu *)

PROCEDURE READLINE (var line : longstr; var linelength : integer);
(***************************************************************************
* Reads one line from the temporary file and assigns it to long string array
***************************************************************************)
var i : integer; (* index *)
    len1,len2,len3,len4 : byte; (* lengths of line1, line2, line3, line4 *)
    line1,line2,line3,line4 : string[255];
          (* read long line from temporary file in parts *)

begin
 (* read in one record of data at a time *)
 repeat   (* skip blank lines *)
  read(tempfile, line1);
  if line1 = '' then
   readln;
 until line1 <> '';
 readln(tempfile,line2,line3,line4);
 len1 := length(line1);
 len2 := length(line2);
 len3 := length(line3);
 len4 := length(line4);
 linelength := len1 + len2 + len3 + len4;

 (* assign line segments to one long string *)
 for i := 1 to len1 do
  line[i] := line1[i];
 if len2 <> 0 then
  begin
   for i := 1 to len2 do
    line[i+255] := line2[i];
   if len3 <> 0 then
    begin
     for i := 1 to len3 do
      line[i+510] := line3[i];
     if len4 <> 0 then
      begin
       for i := 1 to len4 do
        line[i+765] := line4[i];
      end; (* if len4 <> 0 *)
    end; (* if len3 <> 0 *)
  end; (* if len2 <> 0 *)
end; (* readline *)

PROCEDURE READDATA (n : integer; var fileout, tempfile : text);
(***************************************************************************
* Reads references from temporary file and reformats for database file
***************************************************************************)
var author_found : boolean; (* flag: end of author found *)
    ctr          : integer; (* counts length of title field *)
    firstpass    : boolean; (* true on first pass through the loop *)
    i,j,k,l    : integer; (* faithful index friends *)
    line         : longstr;
    linelength   : integer; (* length of line (len1+len2+len3+len4) *)
    nextspace    : integer; (* counter for locating next space in title *)
    remainder    : integer; (* length of title/citation if no colon *)
    title_found  : boolean; (* flag: end of title found *)
    year_found   : boolean; (* flag: end of year found *)

begin (* readdata *)
 Initialize(line);   (* initialize the record *)

 author_found := false;   (* initialize flags *)
 year_found := false;
 title_found := false;
 firstpass := true;

 Odometer(n, 42, 'REWRITING REFERENCES INTO DATABASE FILE',
          'REWRITING REFERENCE NUMBER ');
 ReadLine(line, linelength);


{ write(fileout, '"', n:1, '","'); (* writes record number, if needed *)  }
{WRITELN(fileout,'LINELENGTH=',LINELENGTH:1,'--');}
 write(fileout, '"');
 (* break the reference into database fields *)
 for i := 1 to linelength do
  begin
   if i >= 7 then
    begin (* if *)

     (* pick out the author *)
     if ((line[i] = '9') or (line[i] = '8')) and
         (line[i-1]='1') then
      while not author_found do
       begin (* while *)

        (* find author if date & author separated by ', ' *)
        if (line[i-2] = ' ') and (line[i-3] = ',') then
         for j := 1 to i-4 do
          write(fileout, line[j]);

        (* find author if date & author separated only by ' ' *)
        if (line[i-2] = ' ') and (line[i-3] <> ',') then
         for j := 1 to i-3 do
          write(fileout, line[j]);

        write(fileout, '","');
        author_found := true;
       end; (* while not author_found--pick out the author *)

     (* pick out the year *)
     if not year_found then
      if ((line[i-2] = '9') or (line[i-2] = '8')) and
          (line[i-3] = '1') then
       begin
        (* find comma at end of date *)
        k := i;
        repeat
         k := k + 1;
        until line[k] = ',';
        year_found := true;
        for j := i-3 to k-1 do
         write(fileout, line[j]);
        write(fileout, '","');
        k := k + 2; (* increment k for finding start of title *)
       end; (* pick out the year *)

     (* pick out the title *)
     if not title_found then
      if line[i] = ':' then
       begin
        title_found := true;
        ctr := 0;
        for j := k to i-1 do
         begin
          ctr := ctr + 1;
          if ((ctr > 235) and (ctr <= 256)) and (line[j] = ' ') then
           begin
            nextspace := 0;
            repeat
             nextspace := nextspace + 1;
            until line[j+nextspace] = ' ';
            if nextspace + ctr <= 256 then
             write(fileout, line[j])
            else
              write(fileout, '","', line[j]);
           end (* if ctr > 235, etc. *)
          else
          (* if not (((ctr > 235) and (ctr > 256)) and (line[j] = ' ')) *)
           write(fileout, line[j]);
         end; (* for j := k to i-1 *)

        (* if the title field length < 255 add space for another column,
           just in case there are some titles that extend past 255
           characters into a second column *)
        if ctr < 255 then
         write(fileout, '"," ');

        write(fileout, '","'); (* delimit end of title data *)
        l := i+2; (* l is start of citation in line *)
       end; (* if line[i] = ':'--pick out the title *)

    end; (* if i > 7 *)
  end; (* for i := 1 to linelength *)

 (* if no colon in title or citation, write rest
    of reference out into title and citation columns *)
 if not title_found then
  begin
   ctr := 0;
   remainder := linelength - k + 1;
   for j := k to linelength do
    begin
     ctr := ctr + 1;
     if ((ctr > 235) and (ctr <= 256)) and (line[j] = ' ') then
      begin
       nextspace := 0;
       repeat
        nextspace := nextspace + 1;
       until line[j+nextspace] = ' ';

       if (nextspace + ctr <= 256) then
        begin
         if j <> linelength then
          write(fileout, line[j])
         else (* if j = linelength *)
          if line[j] <> ' ' then
           write(fileout, line[j]);
        end
       else
        begin
         write(fileout, '","', line[j]);
         ctr := 0;
        end;
      end (* if ctr > 235, etc. *)

     else
     (* if not (((ctr > 235) and (ctr <= 256)) and (line[j] = ' ')) *)
      begin
       if j <> linelength then
        write(fileout, line[j])
       else (* if j = linelength *)
        if line[j] <> ' ' then
         write(fileout, line[j]);
      end;

    end; (* for j := k to linelength *)

   (* allow for 2 title fields and 1 citation field *)
   if remainder < 255 then
    write(fileout, '"," "," ');
   if (remainder > 255) and (remainder < 510) then
    write(fileout, '"," ');
  end (* if not title_found *)

 (* if colon was found to end the title, write out the citation *)
 else (* if title_found *)
  for j := l to linelength do
   if not firstpass then
    begin
     if j <> linelength then
      write(fileout, line[j])
     else (* if j = linelength *)
      if line[j] <> ' ' then
       write(fileout, line[j]);
    end
   else (* if firstpass *)
    begin
     if line[j] <> ' ' then
      write(fileout, line[j]);
     firstpass := false;
    end;

 writeln(fileout, '"') (* end the citation field *)

end; (* readdata *)

PROCEDURE OPENFILES;
(***************************************************************************
* Prompts user for input and output file names; opens data files
***************************************************************************)
begin
 (* prompt user for and read in input and output file names *)
 writeln('ENTER NAME OF INPUT DATA FILE (an ascii file)');
 write('   Input file?  ');
 infile := Validname;
 writeln;
 writeln('ENTER NAME OF OUTPUT DATA FILE (delimited database file)');
 write('   Output file?  ');
 outfile := Validname;
 for i := 1 to length(outfile) do
  outfile[i] := UpCase(outfile[i]);

 assign(filein,infile);  (* open input data file *)
 reset(filein);

 assign(tempfile,'temp1.dat');  (* open temporary data file *)
 rewrite(tempfile);

 assign(fileout,outfile);  (* open output database file *)
 rewrite(fileout);
end; (* openfiles *)

PROCEDURE CLOSEFILES;
(***************************************************************************
* Closes data files
***************************************************************************)
begin
 close(filein);
 close(tempfile);
 close(fileout);
 Erase(tempfile);
end; (* closefiles *)

(**************************************************************************)
var menuitem : char;
Begin  (* refs2db *)

 (* print information message *)
 ClrScr;  GotoXY(1,6);
 writeln('WELCOME to Program Refs2DB--');
 writeln('  this program will read bibliographic text from an ascii file');
 writeln('  in paragraph format and write out a delimited file that can be');
 writeln('  imported into PARADOX or DBASE.');
 writeln;
 OpenFiles;

 (* read references from original ASCII file and
    output single line records in temp1.dat *)
 ClrScr;
 number := 0;
 menuitem := Menu;
 case menuitem of
   '1' : ReadHanging(filein,tempfile,number);
   '2' : ReadFlush(filein,tempfile,number);
   '3' : ReadStandard(filein,tempfile,number);
  end; (* case *)
 GotoXY(7,14);
 write('TOTAL NUMBER OF REFERENCES READ FROM ORIGINAL FILE:  ', number:1);
 Pause(7,21,'Press any key to continue');

 (* reset temporary file for read access *)
 reset(tempfile);

 (* rewrite references into database file *)
 ClrScr;
 for n := 1 to number do
  begin
   ReadData(n,fileout,tempfile);
   if n = number then
    begin
     GotoXY(7,14);
     write('NUMBER OF REFERENCES REWRITTEN TO DATABASE FILE:  ', n:1);
     GotoXY(7,16);
     write('==> YOUR DATABASE FILE, "', outfile,'", IS READY');
     Pause(7,21,'Press any key to exit to DOS');
    end; (* if n = number *)
  end;

 CloseFiles;
 ClrScr;
End.  (* refs2db *)
